/*

*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its center, with its center at the origin.
<p>
   Do this by "accumulating" rotations.
<p>
   The last animation accumulated the rotations on the right,
   but in a complicated way. This animation accumulates the
   rotations on the left, which is simpler to implement.
   <pre>{@code
      M = Rz(5*i) * T(-0.5, -0.5, 0)
        = Rz(5) * Rz(5*(i-1)) * T(-0.5, -0.5, 0).
          \_________________/
         accumulate on the left
   }</pre>
*/
public class Example_09d
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene();
      // Since this is a 2D example, we can use an orthographic camera.
      final double right = 2.0;
      final double left = -right;
      final double top = right;
      final double bottom = -top;
      final double near = -1.0;
      scene.getCamera().projOrtho(left, right, bottom, top, near);

      // Create a FrameBuffer to render our scene into.
      final int width  = 512;
      final int height = 512;
      final FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      final Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      final Position axes_p = new Position(axes);
      scene.addPosition(axes_p);

      // Add a square to the Scene.
      final Model square = new Square("Example_09a");
      final Position square_p = new Position(square);
      scene.addPosition(square_p);

      // Push the square away from where the camera is.
      square_p.setMatrix( Matrix.translate(0, 0, -2) );
      // Translate the square's center to the origin, (0, 0).
      square_p.getMatrix().mult( Matrix.translate(-0.5, -0.5, 0) );

      // Rotate (on the left) the translated square around the origin.
      for (int i = 0; i < 72; ++i)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_09d_Frame%03d.ppm", i));

         // Accumulate a rotation on the left of the model matrix.
         square_p.getMatrix().multLeft( Matrix.rotateZ(5) );
       //square_p.setMatrix( Matrix.rotateZ(5).times(square_p.getMatrix()) );
      }
   }
}
