/*

*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Translate the letter {@link P} along the x-axis while
   rotating around the y-axis.
<p>
   See the animation <a href="../Example_23.gif">Example_23.gif</a>.
<p>
   <a href="../Example_23.gif"><img src="../Example_23.gif" alt="Example_23.gif"></a>
 <p>
   This is a harder problem, and this is a wrong way to try to solve
   the problem. This code assumes that rotation around the y-axis
   commutes with translation along the x-axis. But rotations and
   translations only commute when they are along the same axis.
*/
public class Example_23a
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      // Set up a perspective camera.
      final double right = 5.0;
      final double left = -1.0;
      final double top = 2.0;
      final double bottom = -top;
      final double near = 1.0;
      scene.getCamera().projPerspective(left, right, bottom, top, near);

      // Create a FrameBuffer to render our scene into.
      int width  = 600;
      int height = 400;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(left, right, bottom, top, 12, 8);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Add the axes to the Scene.
      Position axes_p = new Position(axes);
      scene.addPosition(axes_p);
      // Push the axes away from where the camera is.
      axes_p.setMatrix( Matrix.translate(0, 0, -near) );

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      ModelShading.setColor(modelP, Color.blue);
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);
      // Push the model away from where the camera is.
      modelP_p.setMatrix( Matrix.translate(0, 0, -near) );

      // Translate along the x-axis while rotating around the y-axis.
      // (This is harder. In fact, this code doesnt work, since it
      // assumes that these rotations and translations commute.)
      for (int i = 0; i <= 100; ++i)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_23a_Frame%03d.ppm", i));

         // Translate down the x-axis and then rotate around the y-axis.
         // This does not do what we want.
         modelP_p.getMatrix().mult( Matrix.translate(0.05, 0, 0) ); // Accumulate the translation.
         modelP_p.getMatrix().mult( Matrix.rotateY(18) );           // Accumulate the rotation.
      }
   }
}
