/*

*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Scale the letter {@link P} to four times its original size
   while keeping the top of the {@link P} at the line y = 1 and
   the right hand edge of the {@link P} at the line x = 1.
<p>
   See the animation <a href="../Example_34.gif">Example_34.gif</a>.
<p>
   <a href="../Example_34.gif"><img src="../Example_34.gif" alt="Example_34.gif"></a>
*/
public class Example_34
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-4, +4, -4, +4, 16, 16);
      // Color them red.
      ModelShading.setColor(axes, Color.red);
      // Push the axes away from where the camera is.
      Position axes_p = new Position(axes);
      axes_p.setMatrix( Matrix.translate(0, 0, -4) );
      // Add the axes to the Scene.
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      ModelShading.setColor(modelP, Color.blue);
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Scale the letter P to four times its original size while keeping
      // the top of the P at the line y=1 and the right hand edge of the
      // P at the line x=1.
      for (int i = 0; i <= 120; i++)
      {
         // Push the model away from where the camera is.
         modelP_p.setMatrix( Matrix.translate(0, 0, -4) );

         // translate then scale
         double scale = 1.0 + i/40.0;
         modelP_p.getMatrix().mult( Matrix.translate(1 - scale, 1 - scale, 0) );
         modelP_p.getMatrix().mult( Matrix.scale(scale, scale, 1) );
/*
         // scale then translate
         double scale = 1.0 + i/40.0;
         modelP_p.getMatrix().mult( Matrix.scale(scale, scale, 1) );
         modelP_p.getMatrix().mult( Matrix.translate(1/scale - 1, 1/scale - 1, 0) );
*/
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_34_Frame%03d.ppm", i));
      }
   }
}
